<?php

/**
 * Cprocessing Payment Admin Controller
 */
class ControllerExtensionPaymentCprocessing extends Controller
{

    /** @var array $error Validation errors */
    private $error = array();

    /** @var boolean $ajax Whether the request was made via AJAX */
    private $ajax = false;

    /** @var CprocessingLibrary $cprocessing */
    private $cprocessing;

    /**
     * Cprocessing Payment Admin Controller Constructor
     * @param Registry $registry
     */
    public function __construct($registry)
    {
        parent::__construct($registry);

        // Make langauge strings and Cpypto-Processing Library available to all
        $this->load->language('extension/payment/cprocessing');

        $this->cprocessing = new Cprocessing($registry);
    }

    /**
     * Primary settings page
     * @return void
     */
    public function index()
    {
        // Saving settings
        if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->request->post['action'] === 'save' && $this->validate()) {
            $this->session->data['success'] = $this->language->get('text_success');
            $this->setting('geo_zone_id', $this->request->post['payment_cprocessing_geo_zone_id']);
            $this->setting('status', $this->request->post['payment_cprocessing_status']);
            $this->setting('sort_order', $this->request->post['payment_cprocessing_sort_order']);
            $this->setting('callback_secret', $this->request->post['payment_cprocessing_callback_secret']);
            $this->setting('callback_url', $this->request->post['payment_cprocessing_callback_url']);
            $this->setting('api_key_btc', $this->request->post['payment_cprocessing_api_key_btc']);
            $this->setting('api_key_bnb', $this->request->post['payment_cprocessing_api_key_bnb']);
            $this->setting('complete_status', $this->request->post['payment_cprocessing_complete_status']);
            if(isset($this->request->post['payment_cprocessing_btc_on'])) {
                $this->setting('btc_on', 1);
            } else {
                $this->setting('btc_on', 0);
            }
            if(isset($this->request->post['payment_cprocessing_bnb_on'])) {
                $this->setting('bnb_on', 1);
            } else {
                $this->setting('bnb_on', 0);
            }
            $this->cprocessing->setCallbackUrl();
            $this->session->data['success'] = $this->language->get('text_success');
            $this->response->redirect($this->url->link('extension/payment/cprocessing', 'user_token=' . $this->session->data['user_token'], true));
        }

        $this->document->setTitle($this->language->get('heading_title'));

        $data['text_statuses'] = $this->language->get('text_statuses');
        $data['text_gen_secret'] = $this->language->get('text_gen_secret');

        $data['entry_geo_zone'] = $this->language->get('entry_geo_zone');
        $data['entry_status'] = $this->language->get('entry_status');
        $data['entry_sort_order'] = $this->language->get('entry_sort_order');
        $data['entry_callback_url'] = $this->language->get('entry_callback_url');
        $data['entry_api_key'] = $this->language->get('entry_api_key');
        $data['entry_callback_secret'] = $this->language->get('entry_callback_secret');
        $data['entry_paid_status'] = $this->language->get('entry_paid_status');
        $data['entry_confirmed_status'] = $this->language->get('entry_confirmed_status');
        $data['entry_complete_status'] = $this->language->get('entry_complete_status');

        $data['help_paid_status'] = $this->language->get('help_paid_status');
        $data['help_confirmed_status'] = $this->language->get('help_confirmed_status');
        $data['help_complete_status'] = $this->language->get('help_complete_status');

        $data['button_save'] = $this->language->get('button_save');
        $data['button_cancel'] = $this->language->get('button_cancel');
        $data['button_gen_secret'] = $this->language->get('button_save');

        $data['url_action'] = $this->url->link('extension/payment/cprocessing', 'user_token=' . $this->session->data['user_token'], 'SSL');
        $data['cancel'] = $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true);
        $data['url_reset'] = $this->url->link('extension/payment/cprocessing/reset', 'user_token=' . $this->session->data['user_token'], 'SSL');
        $data['url_gen_secret'] = $this->url->link('extension/payment/cprocessing/gensecret', 'user_token=' . $this->session->data['user_token'], 'SSL');

        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true)
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_payment'),
            'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true)
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link('extension/payment/cprocessing', 'user_token=' . $this->session->data['user_token'], true)
        );

        // #GENERAL
        $data['cprocessing_connection'] = $this->config->get('payment_cprocessing_connection');
        $this->load->model('localisation/geo_zone');
        $data['geo_zones'] = $this->model_localisation_geo_zone->getGeoZones();
        $data['cprocessing_geo_zone_id'] = (isset($this->request->post['cprocessing_geo_zone_id'])) ? $this->request->post['cprocessing_geo_zone_id'] : $this->setting('geo_zone_id');
        $data['cprocessing_status'] = (isset($this->request->post['cprocessing_status'])) ? $this->request->post['cprocessing_status'] : $this->setting('status');
        $data['cprocessing_sort_order'] = (isset($this->request->post['cprocessing_sort_order'])) ? $this->request->post['cprocessing_sort_order'] : $this->setting('sort_order');
        $data['cprocessing_btc_on'] = (isset($this->request->post['cprocessing_btc_on'])) ? $this->request->post['cprocessing_btc_on'] : $this->setting('btc_on');
        $data['cprocessing_bnb_on'] = (isset($this->request->post['cprocessing_bnb_on'])) ? $this->request->post['cprocessing_bnb_on'] : $this->setting('bnb_on');

        // #ADVANCED
        $data['cprocessing_callback_url'] = (isset($this->request->post['cprocessing_callback_url'])) ? $this->request->post['cprocessing_callback_url'] : $this->setting('callback_url');
        $data['cprocessing_callback_secret'] = (isset($this->request->post['cprocessing_callback_secret'])) ? $this->request->post['cprocessing_callback_secret'] : $this->setting('callback_secret');
        $data['cprocessing_api_key_btc'] = (isset($this->request->post['cprocessing_api_key_btc'])) ? $this->request->post['cprocessing_api_key_btc'] : $this->setting('api_key_btc');
        $data['cprocessing_api_key_bnb'] = (isset($this->request->post['cprocessing_api_key_bnb'])) ? $this->request->post['cprocessing_api_key_bnb'] : $this->setting('api_key_bnb');

        // #ORDER STATUSES
        $this->load->model('localisation/order_status');
        $data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();
        $data['cprocessing_paid_status'] = (isset($this->request->post['cprocessing_paid_status'])) ? $this->request->post['cprocessing_paid_status'] : $this->setting('paid_status');
        $data['cprocessing_confirmed_status'] = (isset($this->request->post['cprocessing_confirmed_status'])) ? $this->request->post['cprocessing_confirmed_status'] : $this->setting('confirmed_status');
        $data['cprocessing_complete_status'] = (isset($this->request->post['cprocessing_complete_status'])) ? $this->request->post['cprocessing_complete_status'] : $this->setting('complete_status');

        // #LAYOUT
        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');

        // #NOTIFICATIONS
        $data['error_warning'] = '';
        if (isset($this->error['warning'])) {
            $data['error_warning'] = $this->error['warning'];
        } elseif (isset($this->session->data['warning'])) {
            $data['error_warning'] = $this->session->data['warning'];
            unset($this->session->data['warning']);
        } else {
            $data['error_warning'] = '';
        }

        $data['success'] = '';
        if (isset($this->session->data['success'])) {
            $data['success'] = $this->session->data['success'];
            unset($this->session->data['success']);
        }

        $data['error_callback_url'] = '';
        if (isset($this->error['callback_url'])) {
            $data['error_callback_url'] = $this->error['callback_url'];
        }

        $data['error_api_key_btc'] = '';
        if (isset($this->error['api_key_btc'])) {
            $data['error_api_key_btc'] = $this->error['api_key_btc'];
        }

        $data['error_api_key_bnb'] = '';
        if (isset($this->error['api_key_bnb'])) {
            $data['error_api_key_bnb'] = $this->error['api_key_bnb'];
        }

        $this->response->setOutput($this->load->view('extension/payment/cprocessing', $data));
    }

    /**
     * Attempts to connect to BitPay API
     * @return void
     */
    public function gensecret()
    {
        //Generate callback secret
        $secret = md5(uniqid(rand(), true));

        $default_callback_url = $this->url->link('extension/payment/cprocessing/callback&secret=' . $secret, $this->config->get('config_secure'));
        $default_callback_url = str_replace(HTTP_SERVER, HTTP_CATALOG, $default_callback_url);
        $default_callback_url = str_replace(HTTPS_SERVER, HTTPS_CATALOG, $default_callback_url);

        $this->setting('callback_url', $default_callback_url);
        $this->setting('callback_secret', $secret);

        // Set new callback url
        $this->cprocessing->setCallbackUrl();


        $this->response->redirect($this->url->link('extension/payment/cprocessing', 'user_token=' . $this->session->data['user_token'] . '', true));
    }


    /**
     * Convenience wrapper for cprocessing settings
     *
     * Automatically persists to database on set and combines getting and setting into one method
     * Assumes cprocessing_ prefix
     *
     * @param string $key Setting key
     * @param string $value Setting value if setting the value
     * @return string|null|void Setting value, or void if setting the value
     */
    private function setting($key, $value = null)
    {
        // Set the setting
        if (func_num_args() === 2) {

            return $this->cprocessing->setting($key, $value);
        }

        // Get the setting
        return $this->cprocessing->setting($key);
    }

    /**
     * Validate the primary settings for the Cpypto-Processing extension
     * @return boolean True if the settings provided are valid
     */
    private function validate()
    {

        if (!$this->user->hasPermission('modify', 'extension/payment/cprocessing')) {
            $this->error['warning'] = $this->language->get('warning_permission');
        }

        if (!empty($this->request->post['payment_cprocessing_callback_url']) && false === filter_var($this->request->post['payment_cprocessing_callback_url'], FILTER_VALIDATE_URL)) {
            $this->error['callback_url'] = $this->language->get('error_callback_url');
        }

        if (empty($this->request->post['payment_cprocessing_api_key_btc']) && empty($this->request->post['payment_cprocessing_api_key_bnb'])) {
            $this->error['api_key_btc'] = $this->language->get('error_api_key');
            $this->error['api_key_bnb'] = $this->language->get('error_api_key');
        }

        return !$this->error;
    }

    /**
     * Install the extension by setting up some smart defaults
     * @return void
     */
    public function install()
    {

        $this->load->model('localisation/order_status');
        $order_statuses = $this->model_localisation_order_status->getOrderStatuses();
        $default_paid = null;
        $default_confirmed = null;
        $default_complete = null;

        foreach ($order_statuses as $order_status) {
            if ($order_status['name'] == 'Processing') {
                $default_paid = $order_status['order_status_id'];
            } elseif ($order_status['name'] == 'Processed') {
                $default_confirmed = $order_status['order_status_id'];
            } elseif ($order_status['name'] == 'Complete') {
                $default_complete = $order_status['order_status_id'];
            }
        }

        $this->cprocessing->log("info", $default_paid);
        $this->cprocessing->log("info", $default_confirmed);
        $this->cprocessing->log("info", $default_complete);

        //Generate callback secret
        $secret = md5(uniqid(rand(), true));

        //Delete old settings from previous Cpypto-Processing installation
        $default_callback_url = $this->url->link('extension/payment/cprocessing/callback&secret=' . $secret, $this->config->get('config_secure'));
        $default_callback_url = str_replace(HTTP_SERVER, HTTP_CATALOG, $default_callback_url);
        $default_callback_url = str_replace(HTTPS_SERVER, HTTPS_CATALOG, $default_callback_url);

        $data['default_callback_url'] = $default_callback_url;
        $this->db->query("delete from oc_setting where code='cprocessing'");
        $this->load->model('setting/setting');
        $default_settings = array(
            'payment_cprocessing_token' => null,
            'payment_cprocessing_geo_zone_id' => '0',
            'payment_cprocessing_status' => '0',
            'payment_cprocessing_sort_order' => null,
            'payment_cprocessing_callback_secret' => $secret,
            'payment_cprocessing_callback_url' => $default_callback_url,
            'payment_cprocessing_api_key_btc' => null,
            'payment_cprocessing_api_key_bnb' => null,
            'payment_cprocessing_version' => $this->cprocessing->version,
            'payment_cprocessing_complete_status' => $default_complete,
            'payment_cprocessing_btc_on' => true,
            'payment_cprocessing_bnb_on' => false
        );
        $this->model_setting_setting->editSetting('payment_cprocessing', $default_settings);

        $this->db->query(
            "CREATE TABLE IF NOT EXISTS " . DB_PREFIX . "cprocessing_orders (
            order_id int NOT NULL,
            status int NOT NULL,
            crypto varchar(3) NOT NULL,
            address varchar(191) NOT NULL,
            satoshi varchar(20) NOT NULL,
            currency varchar(3),
            value longtext,
            txid text,
            memo text,
            KEY orderkey (order_id,crypto))"
        );
    }

    /**
     * Uninstall the extension by removing the settings
     * @return void
     */
    public function uninstall()
    {
        $this->load->model('setting/setting');
        $this->model_setting_setting->deleteSetting('payment_cprocessing');
        $this->db->query('DROP TABLE IF EXISTS `' . DB_PREFIX . 'cprocessing_orders`;');
    }
}
