<?php

/**
 * Crypto-Processing Payment Controller
 */
class ControllerExtensionPaymentCprocessing extends Controller
{

    /** @var boolean $ajax Whether the request was made via AJAX */
    private $ajax = false;

    /** @var CprocessingLibrary $cprocessing */
    private $cprocessing;

    /**
     * Crypto-Processing Payment Controller Constructor
     * @param Registry $registry
     */
    public function __construct($registry)
    {
        parent::__construct($registry);

        // Make langauge strings and -Processing Library available to all
        $this->load->language('extension/payment/cprocessing');
        $this->cprocessing = new Cprocessing($registry);

        // Setup logging
        $this->logger = new Log('cprocessing.log');

        // Is this an ajax request?
        if (!empty($this->request->server['HTTP_X_REQUESTED_WITH']) &&
            strtolower($this->request->server['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') {
            $this->ajax = true;
        }
    }

    /**
     * Displays the Payment Method (a redirect button)
     * @return void
     */
    public function index()
    {

        $data['button_confirm'] = $this->language->get('button_confirm');

        if (isset($this->session->data['error_cprocessing'])) {
            unset($this->session->data['error_cprocessing']);
        }

        $data['redirects'] = [];

        if($this->config->get('payment_cprocessing_btc_on')) {
            $data['redirects'][] = [
                'code' => 'BTC',
                'redirect' => $this->url->link('extension/payment/cprocessing/invoice', 'currency=btc', $this->config->get('config_secure'))
            ];
        }

        if($this->config->get('payment_cprocessing_bnb_on')) {
            $data['redirects'][] = [
                'code' => 'BNB',
                'redirect' => $this->url->link('extension/payment/cprocessing/invoice', 'currency=bnb', $this->config->get('config_secure'))
            ];
        }


        if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/extension/payment/cprocessing')) {
            return $this->load->view($this->config->get('config_template') . '/template/extension/payment/cprocessing', $data);
        } else {
            return $this->load->view('extension/payment/cprocessing', $data);
        }
    }

    /**
     * Convenience wrapper for bitpay logs
     * @param string $level The type of log.
     *                      Should be 'error', 'warn', 'info', 'debug', 'trace'
     *                      In normal mode, 'error' and 'warn' are logged
     *                      In debug mode, all are logged
     * @param string $message The message of the log
     * @param int $depth Depth addition for debug backtracing
     * @return void
     */
    public function log($level, $message, $depth = 0)
    {
        $depth += 1;
        $this->cprocessing->log($level, $message, $depth);
    }

    /**
     * Generatew new bitcoin addres and show the invoice
     * @return void
     */
    public function invoice()
    {
        $crypto = $_GET['currency'];
        $this->load->model('checkout/order');

        if (!isset($this->session->data['order_id'])) {
            $this->response->redirect($this->url->link('checkout/cart'));
            return;
        }
        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
        if (false === $order_info) {
            $this->response->redirect($this->url->link('checkout/cart'));
            return;
        }

        $this->document->setTitle($this->language->get('text_title'));

        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/home')
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_basket'),
            'href' => $this->url->link('checkout/cart')
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_checkout'),
            'href' => $this->url->link('checkout/checkout', '', true)
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_invoice'),
            'href' => $this->url->link('extension/payment/cprocessing/invoice')
        );

        if (empty($order_info['currency_code'])) {
            $this->log('error', 'Cannot prepare invoice without `currency_code`');
            throw Exception('Cannot prepare invoice without `currency_code`');
        }

        if (empty($order_info['total'])) {
            $this->log('error', 'Cannot prepare invoice without `total`');
            throw Exception('Cannot prepare invoice without `total`');
        }

        $this->document->addScript('catalog/view/javascript/qrcode.min.js');
        $this->document->addScript('https://js.pusher.com/7.2/pusher.min.js');

        $data['column_left'] = $this->load->controller('common/column_left');
        $data['column_right'] = $this->load->controller('common/column_right');
        $data['content_top'] = $this->load->controller('common/content_top');
        $data['content_bottom'] = $this->load->controller('common/content_bottom');
        $data['footer'] = $this->load->controller('common/footer');
        $data['header'] = $this->load->controller('common/header');

        $data['currency_code'] = $this->config->get('config_currency');
        $price = $this->cprocessing->getPrice($crypto);

        $fiat_amount = $order_info['total'];
        $bits = intval(1.0e8 * $fiat_amount / $price);
        $satoshi_amount = $bits / 1.0e8;
        $data['satoshi_amount'] = $satoshi_amount;
        $data['fiat_amount'] = $fiat_amount;

        $address = $this->cprocessing->genAddress($crypto);
        $data['address'] = $address->address;
        $data['memo'] = $address->memo ?? '';

        $this->cprocessing->log('info', $data['address'], 1);
        $this->cprocessing->log('info', $price, 1);
        $current_time = time();
        $data['orderTimestamp'] = $current_time;
        $order_id = $order_info['order_id'];
        $data['order_id'] = $order_id;

        $data['success_url'] = $this->url->link('checkout/success');
        $data['timeout_url'] = $this->url->link('extension/payment/cprocessing/timeout', $this->config->get('config_secure'));

        $data['crypto'] = strtoupper($crypto);

        if($crypto == 'btc') {
            $crypto_uri = 'bitcoin';
        } else {
            $crypto_uri = 'binancesmartchain';
        }

        $data['crypto_href'] = $crypto_uri . ":" . $data['address'] . "?amount=" . $satoshi_amount;


        if ($data['address'] != "") {
            if($crypto == 'bnb') $bits = $satoshi_amount;

            $this->db->query("INSERT IGNORE INTO " . DB_PREFIX . "cprocessing_orders (order_id, status, crypto, address, satoshi, currency, value, txid, memo) 
            VALUES ('" . (int)$order_id . "', '-1', '" . $crypto . "', '" . $data['address'] . "', '" . $bits . "', '" . $this->config->get('config_currency') . "', '" . $order_info['total'] . "', null, '" . $data['memo'] . "')");
        }

        $this->response->setOutput($this->load->view('extension/payment/cprocessinginvoice', $data));
    }

    /**
     * Shows timeout message
     * @return void
     */
    public function timeout()
    {

        $this->document->setTitle($this->language->get('text_title'));

        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/home')
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_basket'),
            'href' => $this->url->link('checkout/cart')
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_checkout'),
            'href' => $this->url->link('checkout/checkout', '', true)
        );

        $data['column_left'] = $this->load->controller('common/column_left');
        $data['column_right'] = $this->load->controller('common/column_right');
        $data['content_top'] = $this->load->controller('common/content_top');
        $data['content_bottom'] = $this->load->controller('common/content_bottom');
        $data['footer'] = $this->load->controller('common/footer');
        $data['header'] = $this->load->controller('common/header');

        $this->response->setOutput($this->load->view('extension/payment/cprocessingtimeout', $data));
    }

    /**
     * Convenience wrapper for cprocessing settings
     *
     * Automatically persists to database on set and combines getting and setting into one method
     * Assumes 'cprocessing_' prefix
     *
     * @param string $key Setting key
     * @param string $value Setting value if setting the value
     * @return string|null|void Setting value, or void if setting the value
     */
    public function setting($key, $value = null)
    {
        // Set the setting
        if (func_num_args() === 2) {
            return $this->cprocessing->setting($key, $value);
        }

        // Get the setting
        return $this->cprocessing->setting($key);
    }

    /**
     * Success return page
     *
     * Progresses the order if valid, and redirects to OpenCart's Checkout Success page
     *
     * @return void
     */
    /**
     * public function success() {
     * $this->load->model('checkout/order');
     * $order_id = $this->session->data['order_id'];
     * if (is_null($order_id)) {
     * $this->response->redirect($this->url->link('checkout/success'));
     * return;
     * }
     *
     * $order = $this->model_checkout_order->getOrder($order_id);
     * try {
     * $invoice = $this->cprocessing->getInvoice($this->session->data['cprocessing_invoice']);
     * } catch (Exception $e) {
     * $this->response->redirect($this->url->link('checkout/success'));
     * return;
     * }
     *
     * switch ($invoice->getStatus()) {
     * case 'paid':
     * $order_status_id = $this->setting('paid_status');
     * $order_message = $this->language->get('text_progress_paid');
     * break;
     * case 'confirmed':
     * $order_status_id = $this->setting('confirmed_status');
     * $order_message = $this->language->get('text_progress_confirmed');
     * break;
     * case 'complete':
     * $order_status_id = $this->setting('complete_status');
     * $order_message = $this->language->get('text_progress_complete');
     * break;
     * default:
     * $this->response->redirect($this->url->link('checkout/checkout'));
     * return;
     * }
     *
     * // Progress the order status
     * $this->model_checkout_order->addOrderHistory($order_id, $order_status_id);
     * $this->session->data['cprocessing_invoice'] = null;
     * $this->response->redirect($this->url->link('checkout/success'));
     * }
     */

    /**
     * Callback Handler
     * @return void
     */
    public function callback()
    {
        $this->load->model('checkout/order');

        //Get parameters from callback url
        $secret = $_GET['secret'];
        $txid = $_GET['txid'];
        $value = $_GET['value'];
        $status = $_GET['status'];
        $addr = $_GET['addr'];
        $memo = $_GET['memo'] ?? null;
        $this->log('info', 'Callback Handler called');

        if($this->setting('callback_secret') != $secret) {
            die('Invalid secret');
        }

        $sql = $this->db->query("SELECT * FROM " . DB_PREFIX . "cprocessing_orders WHERE `address` = '" . $addr . "' GROUP BY order_id DESC LIMIT 1");
        $order = $sql->row;

        if (!isset($order['order_id'])) {
            $this->log('error', 'Order with address ' . $addr . ' not found in the records.');
            return;
        }

        //Upate order info
        $query = "UPDATE " . DB_PREFIX . "cprocessing_orders SET status='" . (int)$status . "', txid='" . $txid . "' WHERE address ='" . $addr . "' AND order_id='" . $order['order_id'] ."'";
        $this->db->query($query);

        $sql = $this->db->query("SELECT * FROM " . DB_PREFIX . "cprocessing_orders WHERE `address` = '" . $addr . "' GROUP BY order_id DESC LIMIT 1");
        $order = $sql->row;


        if(!is_null($memo) && $memo != $order['memo']) $status = 0;

        if (!is_null($memo)) {
            $network = 'bnb';
        } else {
            $network = 'bitcoin';
        }

        $order_message = '';

        switch ($status) {
            case 0:
            case 1:
                // order status pending
                $order_status_id = '1';
                $order_message = "Waiting for Confirmation on $network network<br>" .
                    "Bitcoin transaction id: $txid <br>" .
                    "You can view the transaction at: <br>" .
                    "<a href='https://blockchair.com/$network/transaction/$txid' target='_blank'>https://blockchair.com/$network/transaction/$txid</a>";
                break;
            case 2:
                if ($value < $order['satoshi']) {
                    $order_status_id = '7'; // 7 = Canceled
                    $order_message = "<b>Warning: Invoice canceled as Paid Amount was less than expected</b><br>";
                } else {
                    $order_status_id = $this->setting('complete_status');
                }
                $order_message .= "$network transaction id: $txid\r" .
                    "You can view the transaction at:\r" .
                    "<a href='https://blockchair.com/$network/transaction/$txid' target='_blank'>https://blockchair.com/$network/transaction/$txid</a>";                // Progress the order status
                break;
            default:
                $this->log('info', 'Status is not paid/confirmed/complete. Redirecting to checkout/checkout');
                $this->response->redirect($this->url->link('checkout/checkout'));
                return;
        }

        // Progress the order status
        $this->model_checkout_order->addOrderHistory($order['order_id'], $order_status_id, $order_message, 1);
    }
}
